@@ -16,9 +16,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include "xf86Wacom.h"
 #include "wcmFilter.h"
@@ -26,50 +24,56 @@
 #include <fcntl.h>
 #include <unistd.h>
 
+struct checkData {
+	dev_t min_maj;
+	const char *source;
+};
+
+static int checkSource(WacomDevicePtr other, void *data)
+{
+	char *device = wcmOptCheckStr(other, "Device", NULL);
+	WacomCommonPtr pCommon;
+	struct checkData *check = data;
+	Bool match = FALSE;
+
+	/* device can be NULL on some distros */
+	if (!device)
+		return -ENODEV;
+
+	free(device);
+
+	pCommon = other->common;
+	if (pCommon->min_maj && pCommon->min_maj == check->min_maj)
+	{
+		char* source = wcmOptCheckStr(other, "_source", "");
+		/* only add the new tool if the matching major/minor
+		 * was from the same source */
+		if (strcmp(check->source, source))
+			match = TRUE;
+		free(source);
+	}
+
+	return match ? 0 : -ENODEV;
+}
+
 /* wcmCheckSource - Check if there is another source defined this device
  * before or not: don't add the tool by hal/udev if user has defined at least
  * one tool for the device in xorg.conf. One device can have multiple tools
  * with the same type to individualize tools with serial number or areas */
-static Bool wcmCheckSource(InputInfoPtr pInfo, dev_t min_maj)
+static Bool wcmCheckSource(WacomDevicePtr priv, dev_t min_maj)
 {
-	int match = 0;
-	InputInfoPtr pDevices = xf86FirstLocalDevice();
+	int nmatch;
+	struct checkData check = {
+		.min_maj = min_maj,
+		.source = wcmOptCheckStr(priv, "_source", ""),
+	};
 
-	for (; !match && pDevices != NULL; pDevices = pDevices->next)
-	{
-		char* device = xf86CheckStrOption(pDevices->options, "Device", NULL);
-
-		/* device can be NULL on some distros */
-		if (!device)
-			continue;
-
-		free(device);
-
-		if (!strstr(pDevices->drv->driverName, "wacom"))
-			continue;
-
-		if (pInfo != pDevices)
-		{
-			WacomCommonPtr pCommon = ((WacomDevicePtr)pDevices->private)->common;
-			char* fsource = xf86CheckStrOption(pInfo->options, "_source", "");
-			char* psource = xf86CheckStrOption(pDevices->options, "_source", "");
-
-			if (pCommon->min_maj &&
-				pCommon->min_maj == min_maj)
-			{
-				/* only add the new tool if the matching major/minor
-				* was from the same source */
-				if (strcmp(fsource, psource))
-					match = 1;
-			}
-			free(fsource);
-			free(psource);
-		}
-	}
-	if (match)
-		xf86Msg(X_WARNING, "%s: device file already in use by %s. "
-			"Ignoring.\n", pInfo->name, pDevices->name);
-	return match;
+	nmatch = wcmForeachDevice(priv, checkSource, &check);
+	if (nmatch > 0)
+		wcmLog(priv, W_WARNING,
+			    "device file already in use. Ignoring.\n");
+	free((char*)check.source);
+	return nmatch;
 }
 
 /* check if the device has been added.
@@ -80,11 +84,11 @@ static Bool wcmCheckSource(InputInfoPtr pInfo, dev_t min_maj)
  * the xorg.conf and is then hotplugged through the server backend (HAL,
  * udev). In this case, the hotplugged one fails.
  */
-int wcmIsDuplicate(const char* device, InputInfoPtr pInfo)
+int wcmIsDuplicate(const char* device, WacomDevicePtr priv)
 {
 	struct stat st;
 	int isInUse = 0;
-	char* lsource = xf86CheckStrOption(pInfo->options, "_source", NULL);
+	char* lsource = wcmOptCheckStr(priv, "_source", NULL);
 
 	/* always allow xorg.conf defined tools to be added */
 	if (!lsource || !strlen(lsource)) goto ret;
@@ -92,8 +96,9 @@ int wcmIsDuplicate(const char* device, InputInfoPtr pInfo)
 	if (stat(device, &st) == -1)
 	{
 		/* can not access major/minor to check device duplication */
-		xf86Msg(X_ERROR, "%s: stat failed (%s). cannot check for duplicates.\n",
-				pInfo->name, strerror(errno));
+		wcmLog(priv, W_ERROR,
+			    "stat failed (%s). cannot check for duplicates.\n",
+			    strerror(errno));
 
 		/* older systems don't support the required ioctl.  let it pass */
 		goto ret;
@@ -102,7 +107,7 @@ int wcmIsDuplicate(const char* device, InputInfoPtr pInfo)
 	if (st.st_rdev)
 	{
 		/* device matches with another added port */
-		if (wcmCheckSource(pInfo, st.st_rdev))
+		if (wcmCheckSource(priv, st.st_rdev))
 		{
 			isInUse = 3;
 			goto ret;
@@ -111,8 +116,8 @@ int wcmIsDuplicate(const char* device, InputInfoPtr pInfo)
 	else
 	{
 		/* major/minor can never be 0, right? */
-		xf86Msg(X_ERROR, "%s: device opened with a major/minor of 0. "
-			"Something was wrong.\n", pInfo->name);
+		wcmLog(priv, W_ERROR,
+			    "device opened with a major/minor of 0. Something was wrong.\n");
 		isInUse = 4;
 	}
 ret:
@@ -123,93 +128,111 @@ ret:
 static struct
 {
 	const char* type;
-	__u16 tool[3]; /* tool array is terminated by 0 */
+	__u16 tool[5]; /* tool array is terminated by 0 */
 } wcmType [] =
 {
 	{ "stylus", { BTN_TOOL_PEN,       0                  } },
 	{ "eraser", { BTN_TOOL_RUBBER,    0                  } },
 	{ "cursor", { BTN_TOOL_MOUSE,     0                  } },
 	{ "touch",  { BTN_TOOL_DOUBLETAP, BTN_TOOL_FINGER, 0 } },
-	{ "pad",    { BTN_FORWARD,        BTN_0,           0 } }
+	{ "pad",    { BTN_FORWARD,        BTN_0,           KEY_CONTROLPANEL, KEY_ONSCREEN_KEYBOARD, 0 } }
 };
 
 /* validate tool type for device/product */
-Bool wcmIsAValidType(InputInfoPtr pInfo, const char* type)
+Bool wcmIsAValidType(WacomDevicePtr priv, const char* type)
 {
-	int j, k, ret = FALSE;
-	WacomDevicePtr priv = (WacomDevicePtr)pInfo->private;
 	WacomCommonPtr common = priv->common;
+	size_t i, j;
 	char* dsource;
+	Bool user_defined;
 
 	if (!type)
 	{
-		xf86Msg(X_ERROR, "%s: No type specified\n", pInfo->name);
+		wcmLog(priv, W_ERROR, "No type specified\n");
 		return FALSE;
 	}
 
-	dsource = xf86CheckStrOption(pInfo->options, "_source", NULL);
+	dsource = wcmOptCheckStr(priv, "_source", NULL);
+	user_defined = !dsource || !strlen(dsource);
+	free(dsource);
 
-	/* walkthrough all types */
-	for (j = 0; j < ARRAY_SIZE(wcmType); j++)
+	for (i = 0; i < ARRAY_SIZE(wcmType); i++)
 	{
-		if (!strcmp(wcmType[j].type, type))
+		if (strcmp(wcmType[i].type, type) == 0)
 		{
-			for (k = 0; wcmType[j].tool[k] && !ret; k++)
-			{
-				if (ISBITSET (common->wcmKeys, wcmType[j].tool[k]))
-				{
-					ret = TRUE;
-
-					/* non GENERIC devices use BTN_TOOL_FINGER for pad */
-					if (common->wcmProtocolLevel != WCM_PROTOCOL_GENERIC)
-					{
-						if (!strcmp(type, "touch") &&
-							wcmType[j].tool[k] == BTN_TOOL_FINGER)
-						    ret = FALSE;
-					}
-				}
-				else if (!dsource || !strlen(dsource)) /* an user defined type */
-				{
-					/* assume it is a valid type */
-					SETBIT(common->wcmKeys, wcmType[j].tool[k]);
-					ret = TRUE;
-				}
-			}
+			break;
 		}
 	}
 
-	if (!ret)
-		xf86Msg(X_ERROR, "%s: Invalid type '%s' for this device.\n",
-			pInfo->name, type);
+	if (i >= ARRAY_SIZE(wcmType))
+	{
+		/* No type with the given name was found. */
+		wcmLog(priv, W_ERROR, "type '%s' is not known to the driver\n", type);
+		return FALSE;
+	}
 
-	free(dsource);
-	return ret;
+	for (j = 0; wcmType[i].tool[j]; j++)
+	{
+		/* Check if the device has this tool among its declared
+		 * keys. If it does, then this is (generally) a valid
+		 * type.
+		 *
+		 * Non-generic devices are an exception to this rule.
+		 * In particular, they set BTN_TOOL_FINGER on the
+		 * pad device. This means that we need to ignore
+		 * BTN_TOOL_FINGER when checking if "touch" is a
+		 * valid type since that would be a false-positive.
+		 * Note that "touch" can still be detected by another
+		 * key, e.g. BTN_TOOL_DOUBLETAP.
+		 */
+		Bool valid = ISBITSET (common->wcmKeys, wcmType[i].tool[j]);
+		Bool bypass = common->wcmProtocolLevel != WCM_PROTOCOL_GENERIC &&
+			      wcmType[i].tool[j] == BTN_TOOL_FINGER &&
+		              strcmp(type, "touch") == 0;
+
+		if (valid && !bypass) {
+			return TRUE;
+		}
+	}
+
+	if (user_defined)
+	{
+		/* This device does not appear to be the right type, but
+		 * maybe the user knows better than us...
+		 */
+		SETBIT(common->wcmKeys, wcmType[i].tool[0]);
+		wcmLog(priv, W_WARNING, "user-defined type '%s' may not be valid\n", type);
+		return TRUE;
+	}
+
+	/* The driver is probably probing to see if this is a valid
+	 * type to associate with this device for hotplug. Let the
+	 * caller know it is invalid, but don't complain in the logs.
+	 */
+	return FALSE;
 }
 
 /* Choose valid types according to device ID. */
-int wcmDeviceTypeKeys(InputInfoPtr pInfo)
+int wcmDeviceTypeKeys(WacomDevicePtr priv)
 {
 	int ret = 1;
-	WacomDevicePtr priv = pInfo->private;
 	WacomCommonPtr common = priv->common;
 
-	priv->common->tablet_id = common->wcmDevCls->ProbeKeys(pInfo);
+	priv->common->tablet_id = common->wcmDevCls->ProbeKeys(priv);
 
 	switch (priv->common->tablet_id)
 	{
 		case 0xF8:  /* Cintiq 24HDT */
 		case 0xF4:  /* Cintiq 24HD */
 			TabletSetFeature(priv->common, WCM_DUALRING);
-			/* fall through */
-
+			_fallthrough_;
 		case 0x34D: /* MobileStudio Pro 13 */
 		case 0x34E: /* MobileStudio Pro 16 */
 		case 0x398: /* MobileStudio Pro 13 */
 		case 0x399: /* MobileStudio Pro 16 */
 		case 0x3AA: /* MobileStudio Pro 16 */
 			TabletSetFeature(priv->common, WCM_LCD);
-			/* fall through */
-
+			_fallthrough_;
 		case 0x357: /* Intuos Pro 2 M */
 		case 0x358: /* Intuos Pro 2 L */
 		case 0x360: /* Intuos Pro 2 M (Bluetooth) */
@@ -231,8 +254,7 @@ int wcmDeviceTypeKeys(InputInfoPtr pInfo)
 		case 0xBC:  /* I4 */
 		case 0xBD:  /* I4 */
 			TabletSetFeature(priv->common, WCM_ROTATION);
-			/* fall through */
-
+			_fallthrough_;
 		/* tablets with touch ring */
 		case 0x17:  /* BambooFun */
 		case 0x18:  /* BambooFun */
@@ -271,6 +293,9 @@ int wcmDeviceTypeKeys(InputInfoPtr pInfo)
 		case 0xC7:  /* DTU1931 */
 		case 0xCE:  /* DTU2231 */
 		case 0xF0:  /* DTU1631 */
+
+		/* Wacom One display tablet */
+		case 0x3A6: /* DTC133 */
 			TabletSetFeature(priv->common, WCM_LCD);
 			break;
 
@@ -282,7 +307,7 @@ int wcmDeviceTypeKeys(InputInfoPtr pInfo)
 		case 0xFA:  /* Cintiq 22HD */
 		case 0x5B:  /* Cintiq 22HDT Pen */
 			TabletSetFeature(priv->common, WCM_LCD);
-			/* fall through */
+			_fallthrough_;
 		case 0xB0:  /* I3 */
 		case 0xB1:  /* I3 */
 		case 0xB2:  /* I3 */
@@ -330,9 +355,12 @@ int wcmDeviceTypeKeys(InputInfoPtr pInfo)
 		case 0x352:/* Cintiq Pro 32 */
 		case 0x37C:/* Cintiq Pro 24 Pen-Only */
 		case 0x390:/* Cintiq 16 */
+		case 0x391:/* Cintiq 22 */
+		case 0x396:/* DTK-1660E */
+		case 0x3AE:/* Cintiq 16 */
+		case 0x3B0:/* DTK-1660E */
 			TabletSetFeature(priv->common, WCM_ROTATION);
-			/* fall-through */
-
+			_fallthrough_;
 		case 0xF6: /* Cintiq 24HDT Touch */
 		case 0x57: /* DTK2241 */
 		case 0x59: /* DTH2242 Pen */
@@ -365,16 +393,9 @@ int wcmDeviceTypeKeys(InputInfoPtr pInfo)
 			break;
 	}
 
-#ifdef INPUT_PROP_DIRECT
-	{
-		int rc;
-		unsigned long prop[NBITS(INPUT_PROP_MAX)] = {0};
+	if (ISBITSET(common->wcmInputProps, INPUT_PROP_DIRECT))
+		TabletSetFeature(priv->common, WCM_LCD);
 
-		rc = ioctl(pInfo->fd, EVIOCGPROP(sizeof(prop)), prop);
-		if (rc >= 0 && ISBITSET(prop, INPUT_PROP_DIRECT))
-			TabletSetFeature(priv->common, WCM_LCD);
-	}
-#endif
 	if (ISBITSET(common->wcmKeys, BTN_TOOL_PEN))
 		TabletSetFeature(priv->common, WCM_PEN);
 
@@ -408,259 +429,101 @@ int wcmDeviceTypeKeys(InputInfoPtr pInfo)
 	return ret;
 }
 
-#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 14
-static InputOption*
-input_option_new(InputOption *list, char *key, char *value)
-{
-	InputOption *new;
-
-	new = calloc(1, sizeof(InputOption));
-	new->key = strdup(key);
-	new->value = strdup(value);
-	new->next = list;
-	return new;
-}
-
 static void
-input_option_free_list(InputOption **opts)
+wcmAddHotpluggedDevice(WacomDevicePtr priv, const char *basename, const char *type,
+		       WacomToolPtr ser)
 {
-	InputOption *tmp = *opts;
-	while(*opts)
-	{
-		tmp = (*opts)->next;
-		free((*opts)->key);
-		free((*opts)->value);
-		free((*opts));
-		*opts = tmp;
-	}
-}
-#endif
-
-/**
- * Duplicate xf86 options, replace the "type" option with the given type
- * (and the name with "$name $type" and convert them to InputOption
- *
- * @param basename Kernel device name for this device
- * @param type Tool type (cursor, eraser, etc.)
- * @param serial Serial number this device should be bound to (-1 for "any")
- */
-static InputOption *wcmOptionDupConvert(InputInfoPtr pInfo, const char* basename, const char *type, int serial)
-{
-	WacomDevicePtr priv = pInfo->private;
-	WacomCommonPtr common = priv->common;
-	pointer original = pInfo->options;
-	WacomToolPtr ser = common->serials;
-	InputOption *iopts = NULL;
 	char *name;
-	pointer options, o;
 	int rc;
 
-#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 12
-	options = xf86OptionListDuplicate(original);
-#else
-	{
-		InputInfoRec dummy;
-
-		memset(&dummy, 0, sizeof(dummy));
-		xf86CollectInputOptions(&dummy, NULL, original);
-		options = dummy.options;
-	}
-#endif
-	if (serial > -1)
-	{
-		while (ser->serial && ser->serial != serial)
-			ser = ser->next;
-
-		if (strlen(ser->name) > 0)
-			rc = asprintf(&name, "%s %s %s", basename, ser->name, type);
-		else
-			rc = asprintf(&name, "%s %d %s", basename, ser->serial, type);
-	}
-	else
+	if (ser == NULL)
 		rc = asprintf(&name, "%s %s", basename, type);
+	else if (strlen(ser->name) > 0)
+		rc = asprintf(&name, "%s %s %s", basename, ser->name, type);
+	else
+		rc = asprintf(&name, "%s %u %s", basename, ser->serial, type);
 
-	if (rc == -1) /* if asprintf fails, strdup will probably too... */
-		name = strdup("unknown");
+	if (rc == -1)
+		return;
 
-	options = xf86ReplaceStrOption(options, "Type", type);
-	options = xf86ReplaceStrOption(options, "Name", name);
-
-	if (serial > -1)
-		options = xf86ReplaceIntOption(options, "Serial", ser->serial);
+	wcmQueueHotplug(priv, name, type, ser ? ser->serial : UINT_MAX);
 
 	free(name);
-
-	o = options;
-	while(o)
-	{
-		iopts = input_option_new(iopts,
-					 xf86OptionName(o),
-					 xf86OptionValue(o));
-		o = xf86NextOption(o);
-	}
-	xf86OptionListFree(options);
-	return iopts;
-}
-
-
-#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 11
-/**
- * Duplicate the attributes of the given device. "product" gets the type
- * appended, so a device of product "Wacom" will then have a product "Wacom
- * eraser", "Wacom cursor", etc.
- */
-static InputAttributes* wcmDuplicateAttributes(InputInfoPtr pInfo,
-					       const char *type)
-{
-	int rc;
-	InputAttributes *attr;
-	char *product;
-
-	attr = DuplicateInputAttributes(pInfo->attrs);
-	rc = asprintf(&product, "%s %s", attr->product, type);
-	free(attr->product);
-	attr->product = (rc != -1) ? product : NULL;
-	return attr;
-}
-#endif
-
-/**
- * This struct contains the necessary info for hotplugging a device later.
- * Memory must be freed after use.
- */
-typedef struct {
-	InputOption *input_options;
-#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 9
-	InputAttributes *attrs;
-#endif
-} WacomHotplugInfo;
-
-/**
- * Actually hotplug the device. This function is called by the server when
- * the WorkProcs are processed.
- *
- * @param client The server client. unused
- * @param closure A pointer to a struct WcmHotplugInfo containing the
- * necessary information to create a new device.
- * @return TRUE to remove this function from the server's work queue.
- */
-static Bool
-wcmHotplugDevice(ClientPtr client, pointer closure )
-{
-	WacomHotplugInfo *hotplug_info = closure;
-	DeviceIntPtr dev; /* dummy */
-
-#if HAVE_THREADED_INPUT
-	input_lock();
-#endif
-
-	NewInputDeviceRequest(hotplug_info->input_options,
-#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 9
-			      hotplug_info->attrs,
-#endif
-			      &dev);
-#if HAVE_THREADED_INPUT
-	input_unlock();
-#endif
-
-	input_option_free_list(&hotplug_info->input_options);
-
-#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 11
-	FreeInputAttributes(hotplug_info->attrs);
-#endif
-	free(hotplug_info);
-
-	return TRUE;
 }
 
 /**
- * Queue the hotplug for one tool/device of the given type.
- * Device has the same options as the "parent" device, type is one of
- * erasor, stylus, pad, touch, cursor, etc.
- * Name of the new device is set automatically to "<device name> <type>".
+ * Attempt to hotplug a tool with a given type.
  *
- * Note that we don't actually hotplug the device here. We store the
- * information needed to hotplug the device later and then queue the
- * hotplug. The server will come back and call the @ref wcmHotplugDevice
- * later.
+ * If the tool does not claim to support the given type, ignore the
+ * request. Otherwise, verify that it is actually valid and then
+ * queue the hotplug.
  *
- * @param pInfo The parent device
- * @param basename The base name for the device (type will be appended)
+ * @param priv The parent device
+ * @param ser The tool pointer
+ * @param basename The kernel device name
+ * @param idflag Tool ID_XXXX flag to try to hotplug
  * @param type Type name for this tool
- * @param serial Serial number this device should be bound to (-1 for "any")
  */
-static void wcmQueueHotplug(InputInfoPtr pInfo, const char* basename, const char *type, int serial)
+static void wcmTryHotplugSerialType(WacomDevicePtr priv, WacomToolPtr ser, const char *basename, int idflag, const char *type)
 {
-	WacomHotplugInfo *hotplug_info;
-
-	hotplug_info = calloc(1, sizeof(WacomHotplugInfo));
-
-	if (!hotplug_info)
-	{
-		xf86Msg(X_ERROR, "%s: OOM, cannot hotplug dependent devices\n", pInfo->name);
+	if (!(ser->typeid & idflag)) {
+		// No need to print an error message. The device doesn't
+		// claim to support this type so there's no problem.
 		return;
 	}
 
-	hotplug_info->input_options = wcmOptionDupConvert(pInfo, basename, type, serial);
-#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 11
-	hotplug_info->attrs = wcmDuplicateAttributes(pInfo, type);
-#endif
-	QueueWorkProc(wcmHotplugDevice, serverClient, hotplug_info);
+	if (!wcmIsAValidType(priv, type)) {
+		wcmLog(priv, W_ERROR, "invalid device type '%s'.\n", type);
+		return;
+	}
+
+	wcmAddHotpluggedDevice(priv, basename, type, ser);
 }
 
 /**
  * Hotplug all serial numbers configured on this device.
  *
+ * @param priv The parent device
  * @param basename The kernel device name
  */
-static void wcmHotplugSerials(InputInfoPtr pInfo, const char *basename)
+static void wcmHotplugSerials(WacomDevicePtr priv, const char *basename)
 {
-	WacomDevicePtr  priv = (WacomDevicePtr)pInfo->private;
 	WacomCommonPtr  common = priv->common;
 	WacomToolPtr    ser = common->serials;
 
 	while (ser)
 	{
-		xf86Msg(X_INFO, "%s: hotplugging serial %d.\n", pInfo->name, ser->serial);
-
-		if (wcmIsAValidType(pInfo, "stylus") &&
-		    (ser->typeid & STYLUS_ID))
-			wcmQueueHotplug(pInfo, basename, "stylus", ser->serial);
-
-		if (wcmIsAValidType(pInfo, "eraser") &&
-		    (ser->typeid & ERASER_ID))
-			wcmQueueHotplug(pInfo, basename, "eraser", ser->serial);
-
-		if (wcmIsAValidType(pInfo, "cursor") &&
-		    (ser->typeid & CURSOR_ID))
-			wcmQueueHotplug(pInfo, basename, "cursor", ser->serial);
+		wcmLog(priv, W_INFO, "hotplugging serial %u.\n", ser->serial);
+		wcmTryHotplugSerialType(priv, ser, basename, STYLUS_ID, "stylus");
+		wcmTryHotplugSerialType(priv, ser, basename, ERASER_ID, "eraser");
+		wcmTryHotplugSerialType(priv, ser, basename, CURSOR_ID, "cursor");
 
 		ser = ser->next;
 	}
 }
 
-void wcmHotplugOthers(InputInfoPtr pInfo, const char *basename)
+void wcmHotplugOthers(WacomDevicePtr priv, const char *basename)
 {
-	int i, skip = 1;
+	Bool skip = TRUE;
 
-	xf86Msg(X_INFO, "%s: hotplugging dependent devices.\n", pInfo->name);
+	wcmLog(priv, W_INFO, "hotplugging dependent devices.\n");
 
         /* same loop is used to init the first device, if we get here we
          * need to start at the second one */
-	for (i = 0; i < ARRAY_SIZE(wcmType); i++)
+	for (size_t i = 0; i < ARRAY_SIZE(wcmType); i++)
 	{
-		if (wcmIsAValidType(pInfo, wcmType[i].type))
+		if (wcmIsAValidType(priv, wcmType[i].type))
 		{
 			if (skip)
-				skip = 0;
+				skip = FALSE;
 			else
-				wcmQueueHotplug(pInfo, basename, wcmType[i].type, -1);
+				wcmAddHotpluggedDevice(priv, basename, wcmType[i].type, NULL);
 		}
 	}
 
-	wcmHotplugSerials(pInfo, basename);
+	wcmHotplugSerials(priv, basename);
 
-        xf86Msg(X_INFO, "%s: hotplugging completed.\n", pInfo->name);
+	wcmLog(priv, W_INFO, "hotplugging completed.\n");
 }
 
 /**
@@ -672,10 +535,9 @@ void wcmHotplugOthers(InputInfoPtr pInfo, const char *basename)
  * This changes the source to _driver/wacom, all auto-hotplugged devices
  * will have the same source.
  */
-int wcmNeedAutoHotplug(InputInfoPtr pInfo, char **type)
+int wcmNeedAutoHotplug(WacomDevicePtr priv, char **type)
 {
-	char *source = xf86CheckStrOption(pInfo->options, "_source", NULL);
-	int i;
+	char *source = wcmOptCheckStr(priv, "_source", NULL);
 	int rc = 0;
 
 	if (*type) /* type specified, don't hotplug */
@@ -689,9 +551,9 @@ int wcmNeedAutoHotplug(InputInfoPtr pInfo, char **type)
 
 	/* no type specified, so we need to pick the first one applicable
 	 * for our device */
-	for (i = 0; i < ARRAY_SIZE(wcmType); i++)
+	for (size_t i = 0; i < ARRAY_SIZE(wcmType); i++)
 	{
-		if (wcmIsAValidType(pInfo, wcmType[i].type))
+		if (wcmIsAValidType(priv, wcmType[i].type))
 		{
 			free(*type);
 			*type = strdup(wcmType[i].type);
@@ -699,15 +561,17 @@ int wcmNeedAutoHotplug(InputInfoPtr pInfo, char **type)
 		}
 	}
 
-	if (!*type)
+	if (!*type) {
+		wcmLog(priv, W_ERROR, "No valid type found for this device.\n");
 		goto out;
+	}
 
-	xf86Msg(X_INFO, "%s: type not specified, assuming '%s'.\n", pInfo->name, *type);
-	xf86Msg(X_INFO, "%s: other types will be automatically added.\n", pInfo->name);
+	wcmLog(priv, W_INFO, "type not specified, assuming '%s'.\n", *type);
+	wcmLog(priv, W_INFO, "other types will be automatically added.\n");
 
 	/* Note: wcmIsHotpluggedDevice() relies on this */
-	pInfo->options = xf86AddNewOption(pInfo->options, "Type", *type);
-	pInfo->options = xf86ReplaceStrOption(pInfo->options, "_source", "_driver/wacom");
+	wcmOptSetStr(priv, "Type", *type);
+	wcmOptSetStr(priv, "_source", "_driver/wacom");
 
 	rc = 1;
 
@@ -716,9 +580,8 @@ out:
 	return rc;
 }
 
-int wcmParseSerials (InputInfoPtr pInfo)
+int wcmParseSerials (WacomDevicePtr priv)
 {
-	WacomDevicePtr  priv = (WacomDevicePtr)pInfo->private;
 	WacomCommonPtr  common = priv->common;
 	char            *s;
 
@@ -727,7 +590,7 @@ int wcmParseSerials (InputInfoPtr pInfo)
 		return 0; /*Parse has been already done*/
 	}
 
-	s = xf86SetStrOption(pInfo->options, "ToolSerials", NULL);
+	s = wcmOptGetStr(priv, "ToolSerials", NULL);
 	if (s) /*Dont parse again, if the commons have values already*/
 	{
 		char* tok = strtok(s, ";");
@@ -745,16 +608,14 @@ int wcmParseSerials (InputInfoPtr pInfo)
 
 			if (nmatch < 1)
 			{
-				xf86Msg(X_ERROR, "%s: %s is invalid serial string.\n",
-					pInfo->name, tok);
+				wcmLog(priv, W_ERROR, "%s is invalid serial string.\n", tok);
 				free(ser);
 				return 1;
 			}
 
 			if (nmatch >= 1)
 			{
-				xf86Msg(X_CONFIG, "%s: Tool serial %d found.\n",
-					pInfo->name, serial);
+				wcmLog(priv, W_CONFIG, "Tool serial %d found.\n", serial);
 
 				ser->serial = serial;
 
@@ -763,22 +624,19 @@ int wcmParseSerials (InputInfoPtr pInfo)
 
 			if (nmatch >= 2)
 			{
-				xf86Msg(X_CONFIG, "%s: Tool %d has type %s.\n",
-					pInfo->name, serial, type);
+				wcmLog(priv, W_CONFIG, "Tool %d has type %s.\n", serial, type);
 				if ((strcmp(type, "pen") == 0) || (strcmp(type, "airbrush") == 0))
 					ser->typeid = STYLUS_ID | ERASER_ID;
 				else if (strcmp(type, "artpen") == 0)
 					ser->typeid = STYLUS_ID;
 				else if (strcmp(type, "cursor") == 0)
 					ser->typeid = CURSOR_ID;
-				else    xf86Msg(X_CONFIG, "%s: Invalid type %s, defaulting to pen.\n",
-					pInfo->name, type);
+				else    wcmLog(priv, W_CONFIG, "Invalid type %s, defaulting to pen.\n", type);
 			}
 
 			if (nmatch == 3)
 			{
-				xf86Msg(X_CONFIG, "%s: Tool %d is named %s.\n",
-					pInfo->name, serial, name);
+				wcmLog(priv, W_CONFIG, "Tool %d is named %s.\n", serial, name);
 				ser->name = strdup(name);
 			}
 			else ser->name = strdup(""); /*no name yet*/
@@ -812,10 +670,9 @@ int wcmParseSerials (InputInfoPtr pInfo)
  * otherwise.
  * @retval True on success or False otherwise.
  */
-Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
+Bool wcmPreInitParseOptions(WacomDevicePtr priv, Bool is_primary,
 			    Bool is_dependent)
 {
-	WacomDevicePtr  priv = (WacomDevicePtr)pInfo->private;
 	WacomCommonPtr  common = priv->common;
 	char            *s;
 	int		i;
@@ -823,17 +680,18 @@ Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 	int		tpc_button_is_on;
 
 	/* Optional configuration */
-	s = xf86SetStrOption(pInfo->options, "Mode", NULL);
+	s = wcmOptGetStr(priv, "Mode", NULL);
 
-	if (s && (xf86NameCmp(s, "absolute") == 0))
-		set_absolute(pInfo, TRUE);
-	else if (s && (xf86NameCmp(s, "relative") == 0))
-		set_absolute(pInfo, FALSE);
+	if (s && (strcasecmp(s, "absolute") == 0))
+		set_absolute(priv, TRUE);
+	else if (s && (strcasecmp(s, "relative") == 0))
+		set_absolute(priv, FALSE);
 	else
 	{
 		if (s)
-			xf86Msg(X_ERROR, "%s: invalid Mode (should be absolute"
-				" or relative). Using default.\n", pInfo->name);
+			wcmLog(priv, W_ERROR,
+				"invalid Mode (should be absolute"
+				" or relative). Using default.\n");
 
 		/* If Mode not specified or is invalid then rely on
 		 * Type specific defaults from initialization.
@@ -851,46 +709,45 @@ Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 	{
 		priv->wheel_default[WHEEL_ABS_UP] = priv->wheel_default[WHEEL2_ABS_UP] = 4;
 		priv->wheel_default[WHEEL_ABS_DN] = priv->wheel_default[WHEEL2_ABS_DN] = 5;
-		set_absolute(pInfo, TRUE);
+		set_absolute(priv, TRUE);
 	}
 
-	s = xf86SetStrOption(pInfo->options, "Rotate", NULL);
+	s = wcmOptGetStr(priv, "Rotate", NULL);
 
 	if (s)
 	{
 		int rotation = ROTATE_NONE;
 
-		if (xf86NameCmp(s, "CW") == 0)
+		if (strcasecmp(s, "CW") == 0)
 			rotation = ROTATE_CW;
-		else if (xf86NameCmp(s, "CCW") ==0)
+		else if (strcasecmp(s, "CCW") ==0)
 			rotation = ROTATE_CCW;
-		else if (xf86NameCmp(s, "HALF") ==0)
+		else if (strcasecmp(s, "HALF") ==0)
 			rotation = ROTATE_HALF;
-		else if (xf86NameCmp(s, "NONE") !=0)
+		else if (strcasecmp(s, "NONE") !=0)
 		{
-			xf86Msg(X_ERROR, "%s: invalid Rotate option '%s'.\n",
-				pInfo->name, s);
+			wcmLog(priv, W_ERROR, "invalid Rotate option '%s'.\n", s);
 			goto error;
 		}
 
 		if (is_dependent && rotation != common->wcmRotate)
-			xf86Msg(X_INFO, "%s: ignoring rotation of dependent"
-					" device\n", pInfo->name);
+			wcmLog(priv, W_INFO, "ignoring rotation of dependent device\n");
 		else
-			wcmRotateTablet(pInfo, rotation);
+			wcmRotateTablet(priv, rotation);
 		free(s);
 	}
 
-	common->wcmRawSample = xf86SetIntOption(pInfo->options, "RawSample",
+	common->wcmRawSample = wcmOptGetInt(priv, "RawSample",
 			common->wcmRawSample);
 	if (common->wcmRawSample < 1 || common->wcmRawSample > MAX_SAMPLES)
 	{
-		xf86Msg(X_ERROR, "%s: RawSample setting '%d' out of range [1..%d]. Using default.\n",
-			pInfo->name, common->wcmRawSample, MAX_SAMPLES);
+		wcmLog(priv, W_ERROR,
+			    "RawSample setting '%d' out of range [1..%d]. Using default.\n",
+			    common->wcmRawSample, MAX_SAMPLES);
 		common->wcmRawSample = DEFAULT_SAMPLES;
 	}
 
-	common->wcmSuppress = xf86SetIntOption(pInfo->options, "Suppress",
+	common->wcmSuppress = wcmOptGetInt(priv, "Suppress",
 			common->wcmSuppress);
 	if (common->wcmSuppress != 0) /* 0 disables suppression */
 	{
@@ -906,49 +763,48 @@ Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 	 * Slightly depressed curve might be 5,0,100,95
 	 * Slightly raised curve might be 0,5,95,100
 	 */
-	s = xf86SetStrOption(pInfo->options, "PressCurve", "0,0,100,100");
+	s = wcmOptGetStr(priv, "PressCurve", "0,0,100,100");
 	if (s && (IsPen(priv) || IsTouch(priv)))
 	{
 		int a,b,c,d;
 		if ((sscanf(s,"%d,%d,%d,%d",&a,&b,&c,&d) != 4) ||
 				!wcmCheckPressureCurveValues(a, b, c, d))
-			xf86Msg(X_CONFIG, "%s: PressCurve not valid\n",
-				pInfo->name);
+			wcmLog(priv, W_CONFIG, "PressCurve not valid\n");
 		else
 			wcmSetPressureCurve(priv,a,b,c,d);
 	}
 	free(s);
 
-	if (xf86SetBoolOption(pInfo->options, "Pressure2K", 0)) {
-		xf86Msg(X_CONFIG, "%s: Using 2K pressure levels\n", pInfo->name);
+	if (wcmOptGetBool(priv, "Pressure2K", 0)) {
+		wcmLog(priv, W_CONFIG, "Using 2K pressure levels\n");
 		priv->maxCurve = 2048;
 	}
 
 	/*Serials of tools we want hotpluged*/
-	if (wcmParseSerials (pInfo) != 0)
+	if (wcmParseSerials (priv) != 0)
 		goto error;
 
 	if (IsTablet(priv))
 	{
 		const char *prop = IsCursor(priv) ? "CursorProx" : "StylusProx";
-		priv->wcmProxoutDist = xf86SetIntOption(pInfo->options, prop, 0);
+		priv->wcmProxoutDist = wcmOptGetInt(priv, prop, 0);
 		if (priv->wcmProxoutDist < 0 ||
 				priv->wcmProxoutDist > common->wcmMaxDist)
-			xf86Msg(X_CONFIG, "%s: %s invalid %d \n",
-				pInfo->name, prop, priv->wcmProxoutDist);
+			wcmLog(priv, W_CONFIG, "%s invalid %d \n",
+				    prop, priv->wcmProxoutDist);
 		priv->wcmSurfaceDist = -1;
 	}
 
-	priv->topX = xf86SetIntOption(pInfo->options, "TopX", 0);
-	priv->topY = xf86SetIntOption(pInfo->options, "TopY", 0);
-	priv->bottomX = xf86SetIntOption(pInfo->options, "BottomX", 0);
-	priv->bottomY = xf86SetIntOption(pInfo->options, "BottomY", 0);
-	priv->serial = xf86SetIntOption(pInfo->options, "Serial", 0);
+	priv->topX = wcmOptGetInt(priv, "TopX", 0);
+	priv->topY = wcmOptGetInt(priv, "TopY", 0);
+	priv->bottomX = wcmOptGetInt(priv, "BottomX", 0);
+	priv->bottomY = wcmOptGetInt(priv, "BottomY", 0);
+	priv->serial = wcmOptGetInt(priv, "Serial", 0);
 
 	tool = priv->tool;
 	tool->serial = priv->serial;
 
-	common->wcmPanscrollThreshold = xf86SetIntOption(pInfo->options, "PanScrollThreshold",
+	common->wcmPanscrollThreshold = wcmOptGetInt(priv, "PanScrollThreshold",
 			common->wcmPanscrollThreshold);
 
 	/* The first device doesn't need to add any tools/areas as it
@@ -964,11 +820,13 @@ Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 
 		if(toollist) /* Already have a tool with the same type/serial */
 		{
-			xf86Msg(X_ERROR, "%s: already have a tool with type/serial %d/%d.\n",
-					pInfo->name, tool->typeid, tool->serial);
+			wcmLog(priv, W_ERROR,
+				    "already have a tool with type/serial %d/%u.\n",
+				    tool->typeid, tool->serial);
 			goto error;
 		} else /* No match on existing tool/serial, add tool to the end of the list */
 		{
+			assert(common->wcmTool != NULL);
 			toollist = common->wcmTool;
 			while(toollist->next)
 				toollist = toollist->next;
@@ -976,21 +834,20 @@ Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 		}
 	}
 
-	common->wcmThreshold = xf86SetIntOption(pInfo->options, "Threshold",
+	common->wcmThreshold = wcmOptGetInt(priv, "Threshold",
 			common->wcmThreshold);
 
-	if (xf86SetBoolOption(pInfo->options, "ButtonsOnly", 0))
+	if (wcmOptGetBool(priv, "ButtonsOnly", 0))
 		priv->flags |= BUTTONS_ONLY_FLAG;
 
 	/* TPCButton on for Tablet PC by default */
-	tpc_button_is_on = xf86SetBoolOption(pInfo->options, "TPCButton",
+	tpc_button_is_on = wcmOptGetBool(priv, "TPCButton",
 					TabletHasFeature(common, WCM_TPC));
 
 	if (is_primary || IsStylus(priv))
 		common->wcmTPCButton = tpc_button_is_on;
 	else if (tpc_button_is_on != common->wcmTPCButton)
-		xf86Msg(X_WARNING, "%s: TPCButton option can only be set "
-			"by stylus.\n", pInfo->name);
+		wcmLog(priv, W_WARNING, "TPCButton option can only be set by stylus.\n");
 
 	/* a single or double touch device */
 	if (TabletHasFeature(common, WCM_1FGT) ||
@@ -1002,14 +859,14 @@ Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 		 * except when touch is supported */
 		common->wcmTouchDefault = 1;
 
-		touch_is_on = xf86SetBoolOption(pInfo->options, "Touch",
+		touch_is_on = wcmOptGetBool(priv, "Touch",
 						common->wcmTouchDefault);
 
 		if (is_primary || IsTouch(priv))
 			common->wcmTouch = touch_is_on;
 		else if (touch_is_on != common->wcmTouch)
-			xf86Msg(X_WARNING, "%s: Touch option can only be set "
-				"by a touch tool.\n", pInfo->name);
+			wcmLog(priv, W_WARNING,
+				    "Touch option can only be set by a touch tool.\n");
 
 		if (TabletHasFeature(common, WCM_1FGT))
 			common->wcmMaxContacts = 1;
@@ -1023,23 +880,23 @@ Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 		int gesture_is_on;
 		Bool gesture_default = TabletHasFeature(priv->common, WCM_LCD) ? FALSE : TRUE;
 
-		gesture_is_on = xf86SetBoolOption(pInfo->options, "Gesture",
+		gesture_is_on = wcmOptGetBool(priv, "Gesture",
 					    gesture_default);
 
 		if (is_primary || IsTouch(priv))
 			common->wcmGesture = gesture_is_on;
 		else if (gesture_is_on != common->wcmGesture)
-			xf86Msg(X_WARNING, "%s: Touch gesture option can only "
-				"be set by a touch tool.\n", pInfo->name);
+			wcmLog(priv, W_WARNING,
+				    "Touch gesture option can only be set by a touch tool.\n");
 
 		common->wcmGestureParameters.wcmTapTime =
-			xf86SetIntOption(pInfo->options, "TapTime",
+			wcmOptGetInt(priv, "TapTime",
 			common->wcmGestureParameters.wcmTapTime);
 	}
 
 	if (IsStylus(priv) || IsEraser(priv)) {
 		common->wcmPressureRecalibration
-			= xf86SetBoolOption(pInfo->options,
+			= wcmOptGetBool(priv,
 					    "PressureRecalibration", 1);
 	}
 
@@ -1052,14 +909,14 @@ Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 
 	for (i=0; i<WCM_MAX_BUTTONS; i++)
 	{
-		char b[12];
+		char b[20];
 		sprintf(b, "Button%d", i+1);
-		priv->button_default[i] = xf86SetIntOption(pInfo->options, b, priv->button_default[i]);
+		priv->button_default[i] = wcmOptGetInt(priv, b, priv->button_default[i]);
 	}
 
 	/* Now parse class-specific options */
 	if (common->wcmDevCls->ParseOptions &&
-	    !common->wcmDevCls->ParseOptions(pInfo))
+	    !common->wcmDevCls->ParseOptions(priv))
 		goto error;
 
 	return TRUE;
@@ -1086,13 +943,12 @@ error:
  * otherwise.
  * @retval True on success or False otherwise.
  */
-Bool wcmPostInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
+Bool wcmPostInitParseOptions(WacomDevicePtr priv, Bool is_primary,
 			     Bool is_dependent)
 {
-	WacomDevicePtr  priv = (WacomDevicePtr)pInfo->private;
 	WacomCommonPtr  common = priv->common;
 
-	common->wcmMaxZ = xf86SetIntOption(pInfo->options, "MaxZ",
+	common->wcmMaxZ = wcmOptGetInt(priv, "MaxZ",
 					   common->wcmMaxZ);
 
 	/* 2FG touch device */
@@ -1104,11 +960,11 @@ Bool wcmPostInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 		int scroll_distance = WCM_SCROLL_DISTANCE_MM * y_res / 1000;
 
 		common->wcmGestureParameters.wcmZoomDistance =
-			xf86SetIntOption(pInfo->options, "ZoomDistance",
+			wcmOptGetInt(priv, "ZoomDistance",
 					 zoom_distance);
 
 		common->wcmGestureParameters.wcmScrollDistance =
-			xf86SetIntOption(pInfo->options, "ScrollDistance",
+			wcmOptGetInt(priv, "ScrollDistance",
 					 scroll_distance);
 	}
 
